#include <YoutubeApi.h>
#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <ArduinoJson.h> // This Sketch doesn't technically need this, but the library does so it must be installed.

#include <ESP8266WebServer.h>
#include "LedControl.h"

// /Users/bruno/Library/Arduino15/packages/esp8266/hardware/esp8266/2.7.1/**
// /Users/bruno/Library/Arduino15/packages/esp8266/tools/**

// Din: 5, Clk:4; Load(CS): 2;
// D1, D2, D4
LedControl lc = LedControl(5, 4, 2, 1);

const char* ssid = "private";
const char* pswd = "nope";
#define API_KEY "lol"  // your google apps API Token
#define CHANNEL_ID "s" // makes up the url of channel

unsigned long api_mtbs = 1000*6; //mean time between api requests
unsigned long api_lasttime;   //last time api request has been done

WiFiClientSecure client;
YoutubeApi api(API_KEY, client);

const int ip = 245;
const int tazer = 4;
ESP8266WebServer server(80);

void setup() {
  //Initialize Serial Communication
  Serial.begin(115200);
  delay(100);
  lc.shutdown(0, false);
  lc.setIntensity(0, 10);
  lc.clearDisplay(0);
  // Initialize WiFi
  setupWifi();
  setupWebpage();
  LDprint("0",false);
  LDprint("0",true);
}
void loop() {
  server.handleClient();
   if (millis() - api_lasttime > api_mtbs)  {
     Serial.println("Reading...");
    if (api.getChannelStatistics(CHANNEL_ID)) {
      String subs = String(api.channelStats.subscriberCount);
      Serial.println(subs);
      LDprint(subs,true);
    } else {
      Serial.println("Fehler");
    }
    api_lasttime = millis();
  }
}
void handle404() {
  String message = "Subsite Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}
void LDprint(String in, bool sub) {
  int pos = 0;
  if(sub){
    pos = 5;
  }
  if(in.length() > 3) {
      for (int i = 0; i < 3; i++)
      {
        lc.setChar(0, i+pos, 'E', false);
      }
      return;
  }
  for (int i = 0; i < 3; i++) {
    lc.setChar(0, i+pos, ' ', false);
  }
  for (int p = 0; p < in.length(); p++) {
      lc.setChar(0, p+pos, in.charAt(in.length()-1-p), false);
  }
}

void handleRoot() {
  if (server.args() > 0) {
    for ( uint8_t i = 0; i < server.args(); i++ ) {
      if (server.argName(i) == "msg") {
        server.send(200, "text/plain", server.arg(i));
      }
      if (server.argName(i) == "players") {
        String pCount = server.arg(i);
        server.send(200, "text/plain", pCount);
        LDprint(pCount,false);
      }
    }
  } else {
    server.send(200, "text/plain", "Working;)");
  }
}

void setupWifi() {
    // Set WiFi to station mode and disconnect from an AP if it was Previously
  // connected
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  WiFi.begin(ssid, pswd);
  WiFi.config(IPAddress(192, 168, 2, ip), IPAddress(192, 168, 2, ip), IPAddress(255, 255, 255, 0), IPAddress(192, 168, 2, 1));
  Serial.println("");
  //Wait for established connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("Connected with Wifi");
  Serial.println("IP Adress:");
  Serial.println(WiFi.localIP());
  Serial.print("Wifi Setup Done");
  client.setInsecure();
}
void setupWebpage() {
  server.on("/", handleRoot);
  server.on("/taze", []() {
    server.send(200, "text/plain", "lets taze ;)");
    digitalWrite(tazer, HIGH);
    delay(100);
    digitalWrite(tazer, LOW);
  });
  server.onNotFound(handle404);
  server.begin();
  Serial.println("Webpage setup done");

}
